module DINEOFtools
using StatsBase
using NCDatasets
using PyPlot
using Dates
using Mustache

"""
    lon, lat, times = load_coords(datafile)

Load the coordinates from `datafile`.

## Examples
```jldoctest
julia> lon, lat, times = load_coords(datafile)
```
"""
function load_coords(datafile::String)
    NCDatasets.Dataset(datafile) do ds
        lon = ds["lon"][:]
        lat = ds["lat"][:]
        times = ds["time"][:]
        return lon, lat, times
    end
end

"""
    U, V, sigma, valex = load_eofs(datafile)

Load the spatial EOFs `U`, the temporal EOFSs `V`, the singular values `sigma`
and the fill value `valex`, stored in `datafile`.

## Examples
```jldoctest
julia> U, V, sigma, valex = load_eof("eof.nc")
```
"""
function load_eofs(datafile::String)
    NCDatasets.Dataset(datafile) do ds
        U = ds["U"][:]
        V = ds["V"][:]
        sigma = ds["Sigma"][:]
        valex = ds["U"].attrib["missing_value"]
        # Replace valex by NaN
        valexind = U .== valex;
        U[valexind] .= NaN;

        return U, V, sigma, valex
    end
end

"""
    mask, ngood = compute_mask(field; threshold, valex)

Compute the spatial mask (`mask`) and the relative data coverage (`ngood`) as
2D-arrays.

Parameters:
`threshold` (default = 0.03): minimum coverage for a pixel to be considered
as good
`valex` (default = -999.9): value used for the missing value attribute

## Example
```jldoctest
julia> mask, ngood = compute_mask(field;  threshold=0.03, valex=-999.9)
```
"""
function compute_mask(field::Array; threshold::Float64=0.03, valex::Float64=-999.9)
    nlon, nlat, ntimes = size(field)
    ngood = Array{Float64, 2}(undef, nlon, nlat)
    mask = zeros(nlon, nlat)
    for ii = 1:nlon
        for jj = 1:nlat
            ngood[ii,jj] = sum(field[ii,jj,:] .!= valex) / ntimes
            if ngood[ii,jj] >= threshold
                mask[ii,jj] = 1.
            end
        end
    end
    return mask, ngood
end

"""
    pctgood = compute_spatial_coverage(field, mask)

Return the percentage of good pixels for each time step

## Example
```jldoctest
julia> pctgood = compute_spatial_coverage(field, mask)
```
"""
function compute_spatial_coverage(field::Array, mask::Array, valex::Float64)::Vector

    ntimes = size(field)[3]
    nsea = sum(mask .== 1)

    pctgood = Vector{Float64}(undef, ntimes)

    # Loop on the times
    for i = 1:ntimes
        ngood = sum(field[:,:,i] .!== valex)
        pctgood[i] = ngood / nsea
    end

    return pctgood;
end

"""
    plot_eof_mode(U, V, lon, lat, times, N, figname)

Plot the Nth spatial and tempoeral EOF and save it in figure `figname`

## Example
```jldoctest
julia> plot_eof_mode(U, V, lon, lat, times, N, figname)
```
"""

function plot_eof_mode(U, V, lon, lat, times, nummode::Int, figname::String="";
                       Uminpct=1., Umaxpct=99.)

    goodvalues = .!(isnan.(U))
    Uflat = U[goodvalues]

    fig = PyPlot.figure(figsize=(20, 10))
    ax = PyPlot.subplot(121)
    PyPlot.title("EOF no. $(nummode)", fontsize=20)

    pcm = PyPlot.pcolormesh(lon, lat, transpose(U), cmap=PyPlot.cm.RdBu_r,
                            vmin=percentile(Uflat, Uminpct), vmax=percentile(Uflat, Umaxpct))
    PyPlot.colorbar(pcm)
    ax = PyPlot.subplot(122)
    PyPlot.plot(times, V, "ko-", markersize=1, linewidth=.2)
    PyPlot.grid(linestyle="--", linewidth=0.25)
    fig.autofmt_xdate()

    if length(figname) > 0
        PyPlot.savefig(figname, dpi=300, bbox_inches="tight")
    end

    PyPlot.show()
    #PyPlot.close()
end

"""
    read_param_file(paramfile)

Read the parameters from `dineof.init` and return a dictionary
with all the parameters

## Examples
```jldoctest
julia> params = read_param_file("dineof.init")
```
"""
function read_param_file(initfile::String="./dineof.init")::Dict

    params = Dict()
    open("./dineof.init", "r") do df
        for lines in eachline(df)
            if !startswith(lines, "!") & (length(lines) > 0)

                linesplit = lstrip(split(lines, "=")[2])

                for p in ["time", "alpha", "Output", "clouds", "EOF.V", "EOF.Sigma"]
                    if startswith(lines, p)
                        params[replace(p, "." => "_")] = linesplit
                        break
                    end
                end

                for p in ["alpha", "toliter", "tol"]
                    if startswith(lines, p)
                        params[p] = parse(Float64, linesplit)
                        break
                    end
                end

                for p in ["data", "mask", "results", "EOF.U"]
                    if startswith(lines, p)
                        params[replace(p, "." => "_")] = match(r"\[(.*)\]", linesplit)[1]
                        break
                    end
                end

                for p in ["numit", "nev", "neini", "ncv", "rec", "nitemax", "eof", "norm", "seed"]
                    if startswith(lines, p)
                        params[p] = parse(Int, linesplit)
                        break
                    end
                end

            end

        end
    end
    return params
end

"""
    check_params(params)

Check the values of the parameters in the dictionary
obtained from `read_param_file`

## Examples
```jldoctest
julia> check_params(params)
```
"""
function check_params(params::Dict)

    if !isfile(params["time"])
        @warn("File $(params["time"]) does not exist")
    end

    for datafiles in split(params["data"], ",")
        if !isfile(replace(datafiles, "'" => ""))
            @warn("File $(datafiles) does not exist")
        end
    end

    if (params["ncv"] <= params["nev"] + 5)
        @warn("You should have ncv > nev+5")
    end

    if (!isdir(params["Output"]))
        @warn("Directory $(params["Output"]) does not exist")
    end

end

"""
    write_param_file(params, initfile)

Write the DINEOF parameters `params` into the init file `initfile`

## Examples
```jldoctest
julia> params = read_param_file("dineof.init")
```
"""
function write_param_file(params::Dict, initfile::String="./dineof.init")

    # Read the init file template
    # TODO: maybe change directory of the template
    param_template = read("./dineof.init.template", String);

    open(initfile, "w") do df
        write(df, Mustache.render(param_template, params))
    end
end

"""
    mean_time_images(times)

Compute the mean and the minimal times between successive images.

## Example
```jldoctest
julia> timevec = [DateTime(2018, 1, 1, 8, 0, 0),
           DateTime(2018, 1, 2, 12, 0, 0),
           DateTime(2018, 1, 5, 6, 0, 0)]
julia> dtmean, dtmin = mean_time_images(times)
(1.9583333333333333, 1.1666666666666667)
```
"""
function mean_time_images(times::Array)
    deltat = [(times[i+1] - times[i]).value for i = 1:length(times)-1]
    # deltat is in milliseconds
    mean_deltat = mean(deltat) / (86400. * 1000.)
    min_deltat = minimum(deltat) / (86400. * 1000.)
    return mean_deltat::Float64, min_deltat::Float64
end

"""
    compute_alpha(timeperiod, p)

Compute the value of the α parameter from the period to be filtered `timeperiod`
and the parameter `p` of the filter.

Note that α returned by the function has T² as units,
where T is the units of `timeperiod`.

## Example
```jldoctest
julia> compute_alpha(5., 3)
0.21108579925487037
```
"""
function compute_alpha(timeperiod::Float64, p::Int)
    alpha = (timeperiod * timeperiod) / (4. * π * π * p)
    return alpha::Float64
end

"""
    compute_alpha_max(times, p)

Compute the maximal value of the α parameter considering the `times` array

Note that α_{max} has units [day²].

## Example
```jldoctest
julia> timevec = [DateTime(2018, 1, 1, 8, 0, 0),
           DateTime(2018, 1, 2, 12, 0, 0),
           DateTime(2018, 1, 5, 6, 0, 0)]
julia> compute_alpha_max(timevec3)
0.6805555555555557
```
"""
function compute_alpha_max(times::Array)
    dtmean, dtmin = mean_time_images(times)
    alphamax = (dtmin * dtmin) / 2
end


end # module
