using DINEOFtools


using Test
using Dates

@testset "EOF reading" begin
    eoffile = "eof_test.nc"
    if !isfile(eoffile)
        @info("Downloading EOF file")
        download("https://dox.ulg.ac.be/index.php/s/JkaUKnyBBw4QIw5/download", eoffile)
    end
    U, V, s, valex = DINEOFtools.load_eofs(eoffile);
    @test size(U) == (40, 37, 12)
    @test size(V) == (551, 12)
    @test length(s) == size(U)[end]
    @test length(s) == size(V)[end]
end

@testset "Mask" begin

    field3d = ones(3, 4, 10)
    valex = -999.9
    field3d[1:2,1,:] .= valex
    field3d[2,3,4:5] .= valex
    mask, ngood = DINEOFtools.compute_mask(field3d)
    @test mask ==  [0.0 1.0 1.0 1.0; 0.0 1.0 1.0 1.0; 1.0 1.0 1.0 1.0]
    @test ngood == [0.0 1.0 1.0 1.0; 0.0 1.0 0.8 1.0; 1.0 1.0 1.0 1.0]

    mask1, ngood1 = DINEOFtools.compute_mask(field3d; valex=9999.)
    @test mask1 == [1.0 1.0 1.0 1.0; 1.0 1.0 1.0 1.0; 1.0 1.0 1.0 1.0]
    @test ngood1 == [1.0 1.0 1.0 1.0; 1.0 1.0 1.0 1.0; 1.0 1.0 1.0 1.0]

    mask2, ngood2 = DINEOFtools.compute_mask(field3d; threshold=0.9)
    @test ngood2 == [0.0 1.0 1.0 1.0; 0.0 1.0 0.8 1.0; 1.0 1.0 1.0 1.0]
    @test mask2 == [0.0 1.0 1.0 1.0; 0.0 1.0 0.0 1.0; 1.0 1.0 1.0 1.0]

end

@testset "Spatial coverage" begin
	field3d = ones(15, 20, 30)
	valex = -999.9
	field3d[:,:,7] .= valex
	field3d[:,:,22] .= valex
	field3d[1:10,1:5,1:2] .= valex
	mask, ngood = DINEOFtools.compute_mask(field3d, valex=valex)
	pctg = DINEOFtools.compute_spatial_coverage(field3d, mask, valex);

	@test length(pctg) == size(field3d)[3]
	@test pctg[1] == 0.8333333333333334
	@test pctg[2] == 0.8333333333333334
	@test pctg[7] == 0.0
	@test pctg[22] == 0.0
	@test pctg[end] == 1.0

end

@testset "param_parsing" begin
	initfile = "dineof.init"
    if !isfile(initfile)
        @info("Downloading init file")
        download("https://dox.ulg.ac.be/index.php/s/moj4bXFa1cyHGEn/download", initfile)
    end
	p0 = DINEOFtools.read_param_file(initfile)

    @test p0["time"] == "'dates.dat'"
    @test p0["data"] == "'2Dbelcolour_region_period_dat.gher'"
    @test p0["nev"] == 100
    @test p0["eof"] == 1
    @test p0["neini"] == 1
    @test p0["rec"] == 1
    @test p0["nitemax"] == 300
    @test p0["tol"] ==  1.0e-8
    @test p0["EOF_V"] == "'eof.nc#V'"
    @test p0["alpha"] == 0.01
    @test p0["EOF_Sigma"] == "'eof.nc#Sigma'"
    @test p0["norm"] == 0
    @test p0["results"] == "'Output/F2Dbelcolour_region_period_datfilled.gher'"
    @test p0["mask"] == "'seacoos2005.avhrr.mask'"
    @test p0["Output"] == "'Output/'"
    @test p0["toliter"] == 0.001
    @test p0["numit"] == 3
    @test p0["seed"] == 243435
    @test p0["ncv"] == 106
    @test p0["EOF_U"] == "'eof.nc#Usst','eof.nc#Uchl'"
end

@testset "mean_time" begin
    timevec = [DateTime(2018, 1, 1, 8, 0, 0), DateTime(2018, 1, 1, 17, 0, 0),
               DateTime(2018, 1, 3, 0, 0, 0), DateTime(2018, 1, 7, 8, 0, 0)];
    deltamean, deltamin = DINEOFtools.mean_time_images(timevec)
    @test deltamean == 2.
    @test deltamin == 0.375
end

@testset "alpha computation" begin
    alpha0 = DINEOFtools.compute_alpha(10., 3)
	alpha1 = DINEOFtools.compute_alpha(1., 5)
	@test alpha0 == 0.8443431970194815
	@test alpha1 == 0.005066059182116889

    timevec = [DateTime(2018, 1, 1, 8, 0, 0),
           DateTime(2018, 1, 2, 12, 0, 0),
           DateTime(2018, 1, 5, 6, 0, 0)]
    alphamax = DINEOFtools.compute_alpha_max(timevec)
    @test alphamax == 0.6805555555555557
end
